# coding: utf-8

"""generate testdata.
"""

__author__ = 'crohaco'
__author_email__ = 'crohaco.net@gmail.com'
__version__ = '2.1.1'
__license__ = 'Apache License 2.0'

from .containers.list import ListContainer
from .containers.dict import DictContainer
from .containers.iter import IterContainer

from .models.list import ListModel
from .models.dict import DictModel

from .patterns.choice import ChoicePattern
from .patterns.pickout import PickoutPattern
from .patterns.cycle import CyclePattern
from .patterns.hashof import HashOfPattern
from .patterns.increment import IncrementPattern
from .patterns.sequence import SequencePattern


from .formatters.string import StringFormatter
from .formatters.json import JsonFormatter
from .formatters.pickle import PickleFormatter
from .formatters.csv import CsvFormatter

from .api.special import BLANK, ESCAPE

CONTAINERS = {
    'list': ListContainer,
    'dict': DictContainer,
    'iter': IterContainer,
}

MODELS = {
    'list': ListModel,
    'dict': DictModel,
}

PATTERNS = {
    'choice': ChoicePattern,
    'pickout': PickoutPattern,
    'cycle': CyclePattern,
    'hashof': HashOfPattern,
    'increment': IncrementPattern,
    'sequence': SequencePattern,
}


FORMATTERS = {
    'string': StringFormatter,
    'json': JsonFormatter,
    'pickle': PickleFormatter,
    'csv': CsvFormatter,
}

try:
    from .container.ordereddict import OrderedDictContainer
    from .model.ordereddict import OrderedDictModel
    CONTAINERS['ordereddict'] = ODictContainer = OrderedDictContainer
    MODELS['ordereddict'] = ODictModel = OrderedDictModel
except ImportError:
    pass

IncPattern = IncrementPattern
SeqPattern = SequencePattern

# Container is ListContainer
Container = ListContainer


def getmodel(params):
    """
    :param container params: structure of the object to be repeated
    :return: model instance.
    """
    try:
        return MODELS[type(params).__name__.lower()](params)
    except KeyError:
        from .exceptions import ModelTypeInvalid
        raise ModelTypeInvalid(ModelTypeInvalid.message)

try:
    del containers, models, patterns, formatters, exceptions, api
except NameError:
    pass
