# coding: utf-8
from random import choice
from .base import BasePattern
from ..api import display


class ChoicePattern(BasePattern):
    """select an element randomly from the choices.
    """
    def __init__(self, choices, callback=None):
        """
        :param choices: list-elements of choices
        :param callback: function to be called at the end of rendering.
        :return: Choice Pattern instance.
        """
        if isinstance(choices, dict):
            self._choices = []
            for value, num in choices.items():
                self._choices += [value] * num
        else:
            self._choices = choices
        self._callbacks = [] if callback is None else [callback]

    def __repr__(self):
        return '<ChoicePattern {0}>'.format(
            display.repr(self._choices, self._limit_repr))

    def __call__(self, *args):
        """render a element by randomly selection

        :param args: any.
        :return: rendered element.
        """
        element = choice(self._choices)
        for callback in self._callbacks:
            element = callback(element)
        return element
