# coding: utf-8
import hashlib

from .base import BasePattern
from ..api import display

try:
    # python 3.x ?
    if bytes is not str:
        def stringify(something):
            return str(something).encode('utf8')
    else:
        # 2.7 or 2.6
        stringify = str
except NameError:
    # 2.5 or older
    stringify = str


class HashOfPattern(BasePattern):
    """generate hash value.
    """
    def __init__(self, subscript, method='md5', stringify=stringify, callback=None):
        """
        :param immutable-obj subscript: record subscript.
        :param str method: hash method type.(md5 / sha1 / sha224 / sha256 / sha384 / sha512)
        :param function stringify: stringify function to give hash method.
        :param function callback: function to be called at the end of rendering.
        :return: Hash Pattern instance.
        """
        self._subscript = subscript
        self._method = method
        self._hash = getattr(hashlib, method.lower())
        self._stringify = stringify
        self._callbacks = [] if callback is None else [callback]

    def __repr__(self):
        info = '{0} of record[{1}]'.format(
            self._method.upper(), self._subscript)
        return '<HashPattern {0}>'.format(
            display.ellipsis(info, self._limit_repr))

    def __call__(self, record, *args):
        """render a element by hash method.

        :param record: record to be written by pattern().
        :param * args: any
        :return: rendered element.
        """
        seed = record[self._subscript]

        element = self._hash(self._stringify(seed)).hexdigest()
        for callback in self._callbacks:
            element = callback(element)
        return element
